#!/usr/bin/env python3
# encoding: utf-8
#
# This file is a part of the LinkAhead Project.
#
# Copyright (C) 2024 Indiscale GmbH <info@indiscale.com>
# Copyright (C) 2024 Henrik tom Wörden <h.tomwoerden@indiscale.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

import os
import tempfile

import jsonschema.exceptions as schema_exc
import pytest
from caosadvancedtools.table_json_conversion.fill_xlsx import (
    _get_path_rows, _get_row_type_column_index, fill_template)
from openpyxl import load_workbook

from .utils import compare_workbooks


def rfp(*pathcomponents):
    """
    Return full path.
    Shorthand convenience function.
    """
    return os.path.join(os.path.dirname(__file__), *pathcomponents)


def fill_and_compare(json_file: str, template_file: str, known_good: str,
                     schema: str = None, custom_output: str = None):
    """Fill the data into a template and compare to a known good.

Parameters:
-----------
schema: str, optional,
  Json schema to validate against.
custom_output: str, optional
  If given, write to this file and drop into an IPython shell.  For development only.
    """
    with tempfile.TemporaryDirectory() as tmpdir:
        outfile = os.path.join(tmpdir, 'test.xlsx')
        assert not os.path.exists(outfile)
        if custom_output is not None:
            outfile = custom_output
        fill_template(data=json_file, template=template_file, result=outfile,
                      validation_schema=schema)
        assert os.path.exists(outfile)
        generated = load_workbook(outfile)  # workbook can be read
    known_good_wb = load_workbook(known_good)
    compare_workbooks(generated, known_good_wb)


def test_detect():
    example = load_workbook(rfp("data/simple_template.xlsx"))
    assert 0 == _get_row_type_column_index(example['Person'])
    assert [1, 2] == _get_path_rows(example['Person'])


def test_fill_xlsx():
    fill_and_compare(json_file=rfp("data/simple_data.json"),
                     template_file=rfp("data/simple_template.xlsx"),
                     known_good=rfp("data/simple_data.xlsx"),
                     schema=rfp("data/simple_schema.json"))
    fill_and_compare(json_file=rfp("data/multiple_refs_data.json"),
                     template_file=rfp("data/multiple_refs_template.xlsx"),
                     known_good=rfp("data/multiple_refs_data.xlsx"),
                     schema=rfp("data/multiple_refs_schema.json"))


def test_errors():
    with pytest.raises(AssertionError) as exc:
        fill_and_compare(json_file=rfp("data/error_simple_data.json"),
                         template_file=rfp("data/simple_template.xlsx"),
                         known_good=rfp("data/simple_data.xlsx"))
    assert "Auric\nSteve" in str(exc.value)
    with pytest.raises(schema_exc.ValidationError) as exc:
        fill_and_compare(json_file=rfp("data/error_simple_data.json"),
                         template_file=rfp("data/simple_template.xlsx"),
                         known_good=rfp("data/simple_data.xlsx"),
                         schema=rfp("data/simple_schema.json"))
    assert exc.value.message == "0.5 is not of type 'integer'"
